<?php

namespace FM\Service;

use FM\Factory\ApontamentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class ApontamentoService {
    protected $ApontamentoFactory;
    protected $environment;
    protected $newCode;
    
    public function __construct (EntityManager $entityManager, ApontamentoFactory $ApontamentoFactory, Environment $Environment, NewCode $newCode) {
        $this->ApontamentoFactory           = $ApontamentoFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->environment                  = $Environment;
        $this->newCode                      = $newCode;
    }
 
    public function setNewRegister($parametros) {

      $NRORG = $this->environment->getNrOrg();
      $NRUSUARIOLOGADO = $this->environment->getCdOperador();

      $todayrRegister = $this->ApontamentoFactory->getTodayRegister($NRUSUARIOLOGADO);
      $qtRegisters = $todayrRegister[0]['QTDAP'];

      $this->connection->beginTransaction();

      try {
        $NRAPONTAMENTO = null;

        if ($qtRegisters == 0) {
          $NRAPONTAMENTO = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);
          $parametrosNewAppointment = array(
              "NRAPONTAMENTO" => $NRAPONTAMENTO,
              "CDOPERADOR" => $this->environment->getCdOperador(),
              "DTAPONTAMENTO" => $parametros['DTAPONTAMENTO'],
              "IDFALTA"       => 'N',
              "IDAPURADO"     => 'N',
              "NRORG"         => $NRORG
          );
          $this->ApontamentoFactory->insertNewAppointment($parametrosNewAppointment);
        }

        if ($NRAPONTAMENTO == null) {
          $nr = $this->ApontamentoFactory->getNRAP($NRUSUARIOLOGADO);
          $NRAPONTAMENTO = $nr[0]['NRAPONTAMENTO'];
        }

        $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

        $lastEntry = $this->ApontamentoFactory->lastEntry($NRUSUARIOLOGADO);
        if (!empty($lastEntry)) {
          $dateEntry = $lastEntry[0]['DTHRENTRADA'];
        } else {
          $dateEntry = null;
        }

        $parametrosNewRegister = array(
              "NRREGISTROHORAS"=> $NRREGISTROHORAS,
              "NRAPONTAMENTO"  => $NRAPONTAMENTO,
              "DTHRENTRADA"    => $parametros['DTHRENTRADA'],
              "IDTIPORESGISTRO"=> $parametros['IDTIPORESGISTRO'],
              "HRTEMPOREGISTRO"=> ($parametros['IDTIPORESGISTRO'] === 'S') ? null: 0,
              "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
              "IDALMOCO"       => $parametros['IDALMOCO'],
              "DTHRSAIDA"      => $parametros['DTHRSAIDA'],
              "NRORG"          => $NRORG,
              "DATEENTRY"      => $dateEntry
          );

          $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);
          $this->connection->commit();

        }  catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function typeRegister() {
      return $this->ApontamentoFactory->typeRegister($this->environment->getCdOperador());
    }

    public function optionLunch($parametros) {

      $qtLunch = $this->ApontamentoFactory->optionLunch($parametros['NRFUNC']);
      $types = $this->ApontamentoFactory->typeRegister($parametros['NRFUNC']);
      return [$qtLunch, $types];
    }

    public function cardsRegisters($param) {
      return $this->ApontamentoFactory->cardsRegisters($param['CDOPERADOR']);
    }

    public function setAbsence($parametros) {

      $this->connection->beginTransaction();
      $NRORG = $this->environment->getNrOrg();

      try {
          $NRAPONTAMENTO = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);
          $paramsAppointment = array(
              "NRAPONTAMENTO" => $NRAPONTAMENTO,
              "CDOPERADOR" => $this->environment->getCdOperador(),
              "DTAPONTAMENTO" => $parametros['DTAPONTAMENTO'],
              "IDFALTA"       => 'S',
              "IDAPURADO"     => 'N',
              "NRORG"         => $NRORG
          );
          $this->ApontamentoFactory->insertNewAppointment($paramsAppointment);
       
          $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

        $parametrosNewRegister = array(
              "NRREGISTROHORAS"=> $NRREGISTROHORAS,
              "NRAPONTAMENTO"  => $NRAPONTAMENTO,
              "DTHRENTRADA"    => null,
              "IDTIPORESGISTRO"=> null,
              "HRTEMPOREGISTRO"=> null,
              "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
              "IDALMOCO"       => 'N',
              "DTHRSAIDA"      => null,
              "NRORG"          => $NRORG,
              "DATEENTRY"      => null,
          );

          $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);

          $this->connection->commit();

        }  catch(\Exception $e) {
          $this->connection->rollback();
          throw $e;
      }
    }

    public function setLastAppointment($parametros) {

      $this->connection->beginTransaction();
      $NRORG = $this->environment->getNrOrg();
      $NRUSUARIOLOGADO = $this->environment->getCdOperador();

      try {
        $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

        $lastEntry = $this->ApontamentoFactory->lastEntry($NRUSUARIOLOGADO);

        if (!empty($lastEntry)) {
          $dateEntry = $lastEntry[0]['DTHRENTRADA'];
        } else {
          $dateEntry = null;
        }

        $parametrosNewRegister = array(
              "NRREGISTROHORAS"=> $NRREGISTROHORAS,
              "NRAPONTAMENTO"  => $parametros['NRAPONTAMENTO'],
              "DTHRENTRADA"    => null,
              "IDTIPORESGISTRO"=> 'S',
              "HRTEMPOREGISTRO"=> null,
              "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
              "IDALMOCO"       => 'N',
              "DTHRSAIDA"      => $parametros['DTHRSAIDA'],
              "NRORG"          => $NRORG,
              "DATEENTRY"      => $dateEntry,
          );

          $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);

          $this->connection->commit();

        }  catch(\Exception $e) {
          $this->connection->rollback();
          throw $e;
      }
    }
}
?>