import {
    Metadata, Http, IDictionary, IEventParam,
} from '@zeedhi/core';
import {
    Button,
    DateRange, LoadingService, Modal, ModalService, SelectMultiple, Text,
} from '@zeedhi/common';
import {
    IterableCarousel,
    IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class ListaTarefasController {
    public modalInformacoes!: Modal;

    public modalVisualizarImagens!: Modal;

    public modalFiltrarTarefas!: Modal;

    public nrChecklistAgend = '';

    // ---- variáveis que monta as informações adicionais de uma tarefa
    public contrato = '';

    public local = '';

    public cliente = '';

    public tarefa = '';

    public data = '';

    public funcionario = '';

    public situacao = '';
    // -----------

    public stylizeStatus({ component }: IEventParam<Text>) {
        if (component.text === 'Pendente') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
        }
        if (component.text === 'Concluído') {
            component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
        }
        if (component.text === 'Em andamento') {
            component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
        if (component.text === 'Cancelado') {
            component.text = "<span  style='background-color: #dee2e6; color: black; font-size: 10px;'>C A N C E L A D O</span>";
        }
        if (component.text === 'Atrasado') {
            component.text = "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
        }
    }

    public async criarModais() {
        this.modalVisualizarImagens = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00001_000_tarefas/modal/fm_00001_001_visualizar_imagens' });
        this.modalFiltrarTarefas = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00001_000_tarefas/modal/fm_00001_001_filtrar_tarefas' });
        this.modalInformacoes = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00001_000_tarefas/modal/fm_00001_001_dados_tarefa' });
    }

    public destruirModais() {
        this.modalVisualizarImagens?.destroy();
        this.modalFiltrarTarefas?.destroy();
        this.modalInformacoes?.destroy();
    }

    public async exibirModalInformacoes({ component }: any) {
        LoadingService.show();
        Http.post('/getTarefasListadas', {
            NRCHECKLISTAGEND: component.name.split('_')[1],
            TIPO: 2,
        }).then((response: IDictionary) => {
            const dados = response.data.data[0];

            this.contrato = (dados.NRCONTRATOFCT === null) ? 'Não há' : dados.NRCONTRATOFCT;
            this.cliente = dados.NMRAZSOCCLIE;
            this.tarefa = dados.NMCHECKLISTAGEND;
            this.local = `${dados.NMLOCALCLIENTE} - ${dados.NMLOCALPAI}`;
            this.data = `${dados.DTAGEND} - ${dados.HORAAGEND}`;
            this.funcionario = dados.NMOPERADOR;
            this.situacao = dados.SITUACAO;

            this.modalInformacoes.show();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public fecharModalInformacoes() {
        this.modalInformacoes.hide();
    }

    public async exibirModalVisualizarImagens({ component }: any) {
        const nr = component.name.split('_')[1];
        this.nrChecklistAgend = nr;
        this.modalVisualizarImagens.show();
    }

    public fecharModalVisualizarImagens() {
        this.modalVisualizarImagens.hide();
    }

    public async exibirModalFiltros() {
        this.modalFiltrarTarefas.show();
    }

    public fecharModalFiltros() {
        this.modalFiltrarTarefas.hide();
    }

    public carregarCardsTarefas({ component }: any) {
        component.datasource.setFilter({
            TIPO: 1,
        });
    }

    public filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-listagem-tarefas');

        iterable.datasource.setFilter({
            CDCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-cliente').value,
            CDOPERADOR: Metadata.getInstance<SelectMultiple>('filtro-operador').value,
            IDSITUACAOAGEND: Metadata.getInstance<SelectMultiple>('filtro-situacao').value,
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('filtro-contratos').value,
            DTHORAGEND: Metadata.getInstance<DateRange>('data').value,
            TIPO: 3,
        }).finally(() => {
            this.fecharModalFiltros();
        });
    }

    public carregarFotosTarefas({ component }: any) {
        const nr = this.nrChecklistAgend;
        const btnApagar = Metadata.getInstance<Button>('botao-apagar-imagens');
        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
                btnApagar.isVisible = false;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
                btnApagar.isVisible = true;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async excluirFoto() {
        const carrosel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-tarefas');
        const img = carrosel.iterable.datasource.data;
        const [deleteData] = img.filter((value: any, index: any) => index === carrosel.currentSlide);

        LoadingService.show();
        Http.post('/excluirFotoTarefas', {
            NRFOTOTAREFAAGEND: deleteData.NRFOTOTAREFAAGEND,
        }).then(() => {
            const data = img.filter((value: any, index: any) => index !== carrosel.currentSlide);
            carrosel.iterable.datasource.updateData(data);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-listagem-tarefas');

        Metadata.getInstance<SelectMultiple>('filtro-cliente').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-operador').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-situacao').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-local').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-contratos').value = null;
        Metadata.getInstance<DateRange>('data').value = null;

        this.carregarCardsTarefas({ component: iterable });
    }
}
